use rs48_lib::{game::Game, grid_displayer::GridDisplayer, prelude::*};
use tokio::{
    io::AsyncWriteExt,
    net::{tcp::OwnedWriteHalf, TcpListener},
};

async fn game(mut write: OwnedWriteHalf) -> Option<()> {
    loop {
        let rules = GameRules::default();
        let mut game = Game::new(rules);
        let mut controller = SimulatedController::new(50, 20);
        let displayer = GridDisplayer::new(1234);
        loop {
            let grid = game.get_board();
            let display = displayer.display(&grid);
            let clear = termion::clear::All;
            let goto = termion::cursor::Goto(1, 1);
            let frame = format!("{clear}{goto}{display}");
            write.write_all(frame.as_bytes()).await.ok()?;

            let movement = controller.next_move(&game).ok()?;
            game.turn(movement).ok()?;
            std::thread::sleep(std::time::Duration::from_millis(500));
        }
    }
}

#[tokio::main]
async fn main() {
    let url = std::env::var("URL").unwrap_or_else(|_| "localhost:8076".to_string());
    let server = TcpListener::bind(url).await.unwrap();
    loop {
        let (stream, address) = server.accept().await.unwrap();
        println!("[server] connection from '{address}'");
        let (_, write) = stream.into_split();
        tokio::spawn(game(write));
    }
}
